#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>

// 此代码依赖开源库libcurl, 请自行下载源码编译并引用，源码地址: https://curl.haxx.se/download.html

const char* ProxyHost = "flow.hailiangip.com";
const int HttpProxyPort = 14223;
const int Socks5ProxyPort = 14224;
const char* DestUrl = "http://www.baidu.com";
const char* OrderId = ""; // 订单号
const char* Pid = "-1"; // 省份id，-1表示随机
const char* Cid = "-1"; // 城市id，-1表示随机
const char* Sip = "0"; // 是否切换ip，0表示自动切换，1表示不能切换，默认0
const char* Uid = ""; // 自定义，dik情况下，相同的uid会尽可能采用相同的ip，可以认为是同一组会话，不填表示每次请求都随机
const char* ZoneId = ""; // 通道ID
const char* PWD = ""; // 订单密码

size_t WriteCallback(void* contents, size_t size, size_t nmemb, char** response) {
    size_t totalSize = size * nmemb;
    *response = static_cast<char*>(realloc(*response, totalSize + 1));
    if (*response) {
        memcpy(*response, contents, totalSize);
        (*response)[totalSize] = '\0';
    }
    return totalSize;
}

void httpProxyWithPassRequest(const char* destUrl, const char* username, const char* proxyPassword) {
    char proxyUrl[256];
    snprintf(proxyUrl, sizeof(proxyUrl), "http://%s:%s@%s:%d", username, proxyPassword, ProxyHost, HttpProxyPort);

    CURL* curl = curl_easy_init();
    if (curl) {
        curl_easy_setopt(curl, CURLOPT_PROXY, proxyUrl);
        curl_easy_setopt(curl, CURLOPT_URL, destUrl);

        char* response = NULL;
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteCallback);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response);

        CURLcode res = curl_easy_perform(curl);
        if (res != CURLE_OK) {
            fprintf(stderr, "Failed to perform HTTP request: %s\n", curl_easy_strerror(res));
        } else {
            long httpCode = 0;
            curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpCode);
            printf("HTTP代理请求状态码: %ld\n", httpCode);
            if (response) {
                printf("HTTP代理请求响应内容: %s\n", response);
                free(response);
            }
        }

        curl_easy_cleanup(curl);
    }
}

void socks5ProxyWithPassRequest(const char* destUrl, const char* username, const char* proxyPassword) {
    char proxyUrl[256];
    snprintf(proxyUrl, sizeof(proxyUrl), "socks5://%s:%s@%s:%d", username, proxyPassword, ProxyHost, Socks5ProxyPort);

    CURL* curl = curl_easy_init();
    if (curl) {
        curl_easy_setopt(curl, CURLOPT_PROXY, proxyUrl);
        curl_easy_setopt(curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS5); // 设置代理类型为SOCKS5

        curl_easy_setopt(curl, CURLOPT_URL, destUrl);

        char* response = NULL;
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteCallback);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response);

        CURLcode res = curl_easy_perform(curl);
        if (res != CURLE_OK) {
            fprintf(stderr, "Failed to perform socks5 request: %s\n", curl_easy_strerror(res));
        } else {
            long httpCode = 0;
            curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpCode);
            printf("socks5代理请求状态码: %ld\n", httpCode);
            if (response) {
                printf("socks5代理请求响应内容: %s\n", response);
                free(response);
            }
        }

        curl_easy_cleanup(curl);
    }
}

int main() {
    const char* username = OrderId;
    char password[256];

    // 默认账密模式
    snprintf(password, sizeof(password), "pwd=%s&pid=%s&cid=%s&uid=%s&sip=%s", PWD, Pid, Cid, Uid, Sip);

    curl_global_init(CURL_GLOBAL_ALL);

    httpProxyWithPassRequest(DestUrl, username, password);
    socks5ProxyWithPassRequest(DestUrl, username, password);

    curl_global_cleanup();

    return 0;
}